import * as React from 'react';
import classNames from 'classnames';
import { isEqual } from 'lodash';

import {KeyboardNavigableList, KeyboardNavigableListProps} from './KeyboardNavigableList';
import {Layout} from './layout';

import gs from '@ladifire-css/sass/index.scss';

const w = 400;

export interface DynamicListProps extends KeyboardNavigableListProps {
  layout: Layout;
  loadPre: any;
  loadPost: any;
  loadAround: any;
  itemToScroll?: string;
  animateOnScroll?: boolean;
  useWindowScroller?: boolean;
  itemLoadingRenderer?: React.ReactElement;
}

export interface DynamicListState {

}

// TODO: check this
////// @ts-ignore
export class DynamicList<T, K> extends KeyboardNavigableList<DynamicListProps, DynamicListState> {
  static displayName = 'DynamicList';
  // static defaultProps = Object.assign( {}, List.defaultProps, {
  //   className: '',
  //   layout: void 0,
  //   loadPre: noop,
  //   loadPost: noop,
  //   loadAround: noop,
  //   itemToScroll: void 0,
  //   animateOnScroll: false,
  // } );
  protected pendingScrollTop: number | boolean;

  constructor( e: DynamicListProps ) {
    super( e );
    this.onScroll = this.onScroll.bind( this );
    this.setHeight = this.setHeight.bind( this );
    this.layout = e ? e.layout : new Layout();
    this.node = null;
    this.scrollTop = this.layout.setContainerHeight( e ? e.height : 0, 0 );
    this.scrollTop = this.layout.setKeys( e ? e.keys : {}, this.scrollTop );

    if (this.props.useWindowScroller) {
      this.scrollTop = (this.props.scrollTop || 0) as number;
    }

    const r = this.layout.getBounds( this.scrollTop, this.scrollTop + e.height ),
      i = r.start,
      s = r.end;
    const o = this.layout.getTops();
    this.state = Object.assign( {}, this.state, {
      start: i,
      end: s,
      tops: o,
    } );
  }

  UNSAFE_componentWillReceiveProps( t: DynamicListProps ) {
    // this.scrollToIfNotVisible( t );
    t.width !== this.props.width && this.layout.heightCache.invalidate();
    t.height !== this.props.height &&
    ( this.scrollTop = this.layout.setContainerHeight( t.height, this.getScrollTop() ) );
    if ( ! isEqual( t.keys, this.props.keys ) ) {
      if ( ! this.props.preventAutoScroll ) {
        isEqual( t.keys, this.props.keys ) ||
        ( this.scrollTop = this.layout.setKeys( t.keys, this.getScrollTop() ) );
      } else {
        this.layout.setKeys( t.keys, this.getScrollTop() );
      }
    }

    this.relayout();
  }

  scrollToIfNotVisible( t: any ) {
    const r = t.itemToScroll,
      a = t.animateOnScroll;
    r &&
    this.scrollToKey( r, {
      lazy: true,
      animate: a,
    } );
  }

  componentDidUpdate() {
    // let a;
    // ( a = super.componentDidUpdate ).call.apply( a, [ this ].concat( arguments ) );
    'number' === typeof this.pendingScrollTop && this.setScrollTop( this.pendingScrollTop );
    this.loadMore();
  }

  componentDidMount() {
    super.componentDidMount();
    'number' === typeof this.pendingScrollTop && this.setScrollTop( this.pendingScrollTop );
    this.loadMore();
  }

  onScroll( r: any ) {
    this.pendingScrollTop = false;
    super.onScroll( r );
  }

  getScrollTop() {
    if ( 'number' === typeof this.scrollTop ) return this.scrollTop;
    if ( 'number' === typeof this.pendingScrollTop ) return this.pendingScrollTop;
    if ( ! this.node ) return 0;

    return super.getScrollTop();
  }

  setHeight( t: string, a: number ) {
    if ( ! t ) return;
    if ( Math.abs( a - this.layout.getHeight( t ) ) < 0.5 ) return;
    const r = this.getScrollTop();
    this.scrollTop = this.layout.setHeight( t, a, r );
    this.relayout();
  }

  loadMore() {
    //if ( this.isScrolling() ) return false;
    const t = arguments.length > 0 && void 0 !== arguments[ 0 ] ? arguments[ 0 ] : this.state.start;
    const a = arguments.length > 1 && void 0 !== arguments[ 1 ] ? arguments[ 1 ] : this.state.end;
    const r = arguments.length > 2 && void 0 !== arguments[ 2 ] ? arguments[ 2 ] : this.props;
    const n = r.keys,
      i = r.height;
    const s = this.layout.getTotalHeight();
    const o = this.layout.getTop( n[ t ] );
    const l = this.layout.getBottom( n[ a - 1 ] );
    const c = ! n.length || s <= i || o < w;
    const u = ! n.length || s <= i || s - l < w;
    // if ( c && u ) return r.loadAround();
    if ( c && typeof r.loadPre === 'function' ) return r.loadPre();
    if ( u && typeof r.loadPost === 'function' ) return r.loadPost();
    return null;
  }

  // renderItem( t: string, r: any ) {
  //   const a = this.getTop( t );
  //   const n = this.layout.getHeight( t );
  //   const i = this.layout.getHeightValidity( t );
  //   const s = this.state.activeItem === t;
  //   const o =
  //     null === this.state.activeItem &&
  //     this.state.hasKeyboardFocus &&
  //     this.state.initialActiveItem === t;
  //   const l = this.state.mouseDownTargetItem === t;
  //   const c = this.state.hoveredItem === t;
  //   const u = this.props.getAriaLabelForRow;
  //   // const p = this.props.isItemAriaExpanded;
  //   const d =
  //     u &&
  //     u( r, {
  //       hasFocus: s,
  //     } );
  //   // const h = p && p( t, a );
  //   return React.createElement(
  //     ListItem,
  //     {
  //       key: t,
  //       id: t,
  //       height: n,
  //       validity: i,
  //       style: {
  //         top: a,
  //       },
  //       role: this.props.itemRole,
  //       hasFocus: s,
  //       hasFocusWithin: o,
  //       isMouseDownOnItem: l,
  //       onHeightChange: this.setHeight,
  //       onFocusEnter: this.onItemFocusEnter,
  //       onFocusWithin: this.onItemFocusWithin,
  //       onMouseOver: this.onItemMouseOver,
  //       onMouseLeave: this.onItemMouseLeave,
  //       onMouseDown: this.onItemMouseDown,
  //       onMouseUp: this.onItemMouseUp,
  //       ariaLabel: d,
  //     },
  //     this.props.rowRenderer( r, {
  //       hasFocus: s,
  //       isHovered: c,
  //     } )
  //   );
  // }

  renderLoading( t: any ) {
    const a = t
      ? {
        top: 0,
        paddingBottom: 8,
        paddingTop: 8,
        position: 'absolute',
      }
      : {
        bottom: 0,
        paddingBottom: 8,
        paddingTop: 8,
        position: 'absolute',
      };
    const r = t ? 'degradedLoadingTop' : 'degradedLoadingBottom';
    return React.createElement(
      'div',
      {
        key: r,
        className: classNames(
          gs.display__flex,
          gs.width__100_percent,
          gs.align_item__center,
          gs.justify_content__center,
          gs.background_color__transparent,
        ),
        style: a,
      },
      React.createElement(
        'div',
        {
          className: 'p-degraded_list__loading_txt',
        },
        'Đang tải thêm...'
      )
    );
  }

  // @ts-ignore
  renderItems(shouldRenderLoading?: boolean = true) {
    const a = super.renderItems();

    const r = this.props.keys.length > 1 && ( ! this.props.reachedStart || this.state.start > 0 );
    const i =
      this.props.keys.length > 1 &&
      ( ! this.props.reachedEnd || this.state.end < this.props.keys.length );

    // TODO: check this
    // @ts-ignore
    return [ r && shouldRenderLoading && this.renderLoading( true ) ].concat( /*n()(a)*/ a, [
      i && shouldRenderLoading && this.renderLoading( false ),
    ] );
  }

  render() {
    this.pendingScrollTop = this.scrollTop;
    this.scrollTop = false;

    return super.render();
  }
}
