import * as React from 'react';

import {classNames} from '@ladifire-ui-react/utils';

import Layout from './horizontal_layout';
import HorizontalItem from './HorizontalListItem';

import gs from '@ladifire-css/sass/index.scss';

export interface HorizontalListProps {
  keys?: string[];
  itemRenderer: (key: any) => JSX.Element;
  onWidthChange: (width: number) => void;
  containerWidth?: number;
  width?: number;
  height?: number;
}

export const HorizontalList = (props: HorizontalListProps) => {
  const {
    keys,
    itemRenderer,
    onWidthChange,
    containerWidth,
    width,
  } = props;

  console.log('containerWidth: ', containerWidth);

  const _layout = new Layout();

  React.useEffect(() => {
    // _layout.invalidate();
    console.log('_layout: ', _layout);
  }, [containerWidth]);

  React.useEffect(() => {
    _layout.setKeys(keys);
  }, [keys]);

  const getLeft = (key: string): number => {
    return _layout.getLeft(key);
  };

  const getTotalWidth = () => {
    return _layout.getTotalWidth();
  };

  const getValidity = (key: string): boolean => {
    return _layout.getWidthValidity(key);
  };

  const setWidth = (key: string, size: number) => {
    console.log('called set width for: ', key, size);
    _layout.setWidth(key, size);
    setTimeout(() => {
      onWidthChange(_layout.getTotalWidth());
    }, 0);
  };

  const renderItem = (key: string) => {
    const _left = getLeft(key);
    const _width = _layout.getWidth(key);
    const _validity = getValidity(key);

    console.log('_left:::+++', _left);
    console.log('_validity:::+++', _validity);

    return (
      <HorizontalItem
        id={key}
        key={key}
        width={_width}
        validity={_validity}
        style={{
          left: _left,
        }}
        onWidthChange={setWidth}
      >
        {itemRenderer(key)}
      </HorizontalItem>
    );
  };

  console.log('width: ', getTotalWidth());

  return (
    <div
      className={classNames(
        gs,
        gs.position__relative,
        gs.overflow_y__hidden,
        gs.overflow_x__hidden,
      )}
      style={{width, height: 200, position: 'relative'}}
    >
      {keys && keys.length && keys.map(renderItem)}
    </div>
  );

  // return (
  //   <div>
  //     {keys && keys.length && keys.map(renderItem)}
  //   </div>
  // );
};
